/** @file
  Logo DXE Driver, install Edkii Platform Logo protocol.

Copyright (c) 2016 - 2017, Intel Corporation. All rights reserved.<BR>
This program and the accompanying materials
are licensed and made available under the terms and conditions of the BSD License
which accompanies this distribution.  The full text of the license may be found at
http://opensource.org/licenses/bsd-license.php

THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.

**/


#include <Uefi.h>
#include <Library/UefiLib.h>
#include <Library/ShellCEntryLib.h>
#include <Protocol/HiiDatabase.h>
#include <Protocol/GraphicsOutput.h>
#include <Protocol/HiiImageEx.h>
#include <Protocol/PlatformLogo.h>
#include <Protocol/HiiPackageList.h>
#include <Library/UefiBootServicesTableLib.h>
#include <Library/DebugLib.h>
#include <Library/PrintLib.h>
#include <Library/MemoryAllocationLib.h>
#include <Library/BaseLib.h>

#include <Library/UefiApplicationEntryPoint.h>

#define EFI_GRAPHICS_OUTPUT_PROTOCOL_GUID \
  { \
    0x9042a9de, 0x23dc, 0x4a38, {0x96, 0xfb, 0x7a, 0xde, 0xd0, 0x80, 0x51, 0x6a } \
  }
  
static EFI_GUID GraphicsOutputProtocolGuid = EFI_GRAPHICS_OUTPUT_PROTOCOL_GUID;

//DO NOT REMOVE IMAGE_TOKEN (IMG_LOGO)

/**
  Entrypoint of this module.

  This function is the entrypoint of this module. It installs the Edkii
  Platform Logo protocol.

  @param  ImageHandle       The firmware allocated handle for the EFI image.
  @param  SystemTable       A pointer to the EFI System Table.

  @retval EFI_SUCCESS       The entry point is executed successfully.

**/
EFI_STATUS
EFIAPI
UefiMain (
  IN EFI_HANDLE        ImageHandle,
  IN EFI_SYSTEM_TABLE  *SystemTable
)
{
  EFI_STATUS                  Status;
  EFI_HII_PACKAGE_LIST_HEADER *PackageListHeader;
  EFI_HII_IMAGE_PACKAGE_HDR   *ImageHeader;
  EFI_HII_IMAGE_BLOCK         *ImageBlocktType;
  EFI_HII_IIBT_IMAGE_24BIT_BASE  *ImageData;
  EFI_GRAPHICS_OUTPUT_PROTOCOL   *GraphicsOutput;
  UINT16        i,j;
  UINT8*        RGB32;
  
        //Step1. Get Package List Header Address
        
        //
        // Retrieve HII package list from ImageHandle
        //
        Status = gBS->OpenProtocol (
                        ImageHandle,
                        &gEfiHiiPackageListProtocolGuid,
                        (VOID **) &PackageListHeader,
                        ImageHandle,
                        NULL,
                        EFI_OPEN_PROTOCOL_GET_PROTOCOL
                        );
                        
        if (EFI_ERROR (Status)) {
          Print(L"HII Image Package with logo not found in PE/COFF resource section\n");
          return Status;
        }
        
        Print(L"PackageList :\n   GUID=[%g] Length=[%X]\n",
                      PackageListHeader->PackageListGuid,
                      PackageListHeader->PackageLength);
     
        //Step2. Parser HII Image
        ImageHeader=(EFI_HII_IMAGE_PACKAGE_HDR*)(PackageListHeader+1);
        Print(L"Image Header:\n Length=[%d]  Type=[%d]\n",ImageHeader->Header.Length,ImageHeader->Header.Type);
        Print(L"ImageOffset %d  PalOffset %d\n",ImageHeader->ImageInfoOffset,ImageHeader->PaletteInfoOffset);
     
        ImageBlocktType=(EFI_HII_IMAGE_BLOCK *)(ImageHeader+1);
        Print(L"ImageBlockType %x\n",*ImageBlocktType);
     
        ImageData=(EFI_HII_IIBT_IMAGE_24BIT_BASE *)(ImageBlocktType+1);
        Print(L"ImageData:\n  Width=[%d]   Heigth=[%d]\n",ImageData->Width,ImageData->Height);

        Status = gBS->LocateProtocol(&GraphicsOutputProtocolGuid, NULL, (VOID **) &GraphicsOutput);
        if (EFI_ERROR(Status)) {
                Print(L"Loading Graphics_Output_Protocol error!\n");
                return EFI_SUCCESS;
        }

        //Step3. Get BMP Image
        RGB32 = (UINT8*)AllocatePool(ImageData->Width *  ImageData->Height *4);
        for (j=0;j<ImageData->Height;j++) {
                for (i=0;i<ImageData->Width;i++) {
                        RGB32[(j*ImageData->Width+i)*4]  =  ImageData->Bitmap[(j*ImageData->Width+i)].b;//Blue   
                        RGB32[(j*ImageData->Width+i)*4+1]=  ImageData->Bitmap[(j*ImageData->Width+i)].g; //Green 
                        RGB32[(j*ImageData->Width+i)*4+2]=  ImageData->Bitmap[(j*ImageData->Width+i)].r; //Red  
                        RGB32[(j*ImageData->Width+i)*4+3]=  0;
                }
        }   
        
        //Step4. Show the Image
        GraphicsOutput->Blt(
                        GraphicsOutput, 
                        (EFI_GRAPHICS_OUTPUT_BLT_PIXEL *) RGB32,
                        EfiBltBufferToVideo,
                        0, 0, 
                        200, 0, 
                        ImageData->Width, ImageData->Height, 0); 
        
        FreePool(RGB32);
        
  return Status;
}
